--[[

This file is part of RampageAI.

RampageAI is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

RampageAI is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with RampageAI; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

--]]


--[[

	****** READ ******
	
	You do not need to edit this file. If you wish to
	change the way your homunculus behaves, please edit
	Conf\Config.lua.
	
	This file checks to make sure you will not get any
	Lua errors in the event that your Config.lua has invalid
	values in it.
	
--]]


--------------------------------
-- Configuration Verification --
--------------------------------

-- Function to verify that all of the configuration options exist
-- (partly so users aren't forced to update their Config.lua constantly)
function CheckConfig()
	-- Keep a log of invalids and out-of-ranges
	local invalids = StringBuffer.New()
	local oorange = StringBuffer.New()

	-- Verify AlchemistDex
	if type(AlchemistDex) ~= "number" then
		StringBuffer.Append(invalids," AlchemistDex")
		AlchemistDex = 30
	else
		if AlchemistDex < 1 then
			StringBuffer.Append(oorange," AlchemistDex")
			AlchemistDex = 1
		elseif AlchemistDex > 150 then
			StringBuffer.Append(oorange," AlchemistDex")
			AlchemistDex = 150
		end
	end

	-- Verify AlchemistPhen
	if type(AlchemistPhen) ~= "boolean" then
		StringBuffer.Append(invalids," AlchemistPhen")
		AlchemistPhen = false
	end

	-- Verify the SightRange variable
	if type(SightRange) ~= "number" then
		StringBuffer.Append(invalids," SightRange")
		SightRange = 12
	else
		-- Check the range of the number
		if SightRange > 15 then
			StringBuffer.Append(oorange," SightRange")
			SightRange = 15
		elseif SightRange < 2 then
			StringBuffer.Append(oorange," SightRange")
			SightRange = 2
		end
	end
	
	-- Verify the MaxHomunRange
	if type(MaxHomunRange) ~= "number" then
		StringBuffer.Append(invalids," MaxHomunRange")
		MaxHomunRange = 12
	else
		if MaxHomunRange > 15 then
			StringBuffer.Append(oorange," MaxHomunRange")
			MaxHomunRange = 15
		elseif MaxHomunRange < 2 then
			StringBuffer.Append(oorange," MaxHomunRange")
			MaxHomunRange = 2
		end
	end
	
	-- Verify HomunRangeChange
	if type(HomunRangeChange) ~= "number" then
		StringBuffer.Append(invalids," HomunRangeChange")
		HomunRangeChange = 4
	else
		-- Verify the range of HomunRangeChange
		if HomunRangeChange >= MaxHomunRange - 2 then
			StringBuffer.Append(oorange," HomunRangeChange")
			HomunRangeChange = MaxHomunRange - 2
		elseif HomunRangeChange < 0 then
			StringBuffer.Append(oorange," HomunRangeChange")
			HomunRangeChange = 0
		end
	end
	
	-- Verify HomunFollowDistance
	if type(HomunFollowDistance) ~= "number" then
		StringBuffer.Append(invalids," HomunFollowDistance")
		HomunFollowdistance = 3
	else
		-- Verify the range of HomunFollowDistance
		if HomunFollowDistance > MaxHomunRange - HomunRangeChange then
			StringBuffer.Append(oorange," HomunFollowDistance")
			HomunFollowDistance = MaxHomunRange - HomunRangeChange - 1
		elseif HomunFollowDistance < 1 then
			StringBuffer.Append(oorange," HomunFollowDistance")
			HomunFollowDistance = 1
		end
	end

	-- Verify HomunRunAhead
	if type(HomunRunAhead) ~= "boolean" then
		StringBuffer.Append(invalids," HomunRunAhead")
		HomunRunAhead = false
	end

	-- Verify AllowIgnoreOwner
	if type(AllowIgnoreOwner) ~= "boolean" then
		StringBuffer.Append(invalids," AllowIgnoreOwner")
		AllowIgnoreOwner = true
	end

	-- Verify HomunMaxMoveDistance
	if type(HomunMaxMoveDistance) ~= "number" then
		StringBuffer.Append(invalids," HomunMaxMoveDistance")
		HomunMaxMoveDistance = 12
	else
		-- Check the range
		if HomunMaxMoveDistance < 2 then
			StringBuffer.Append(oorange," HomunMaxMoveDistance")
			HomunMaxMoveDistance = 2
		elseif HomunMaxMoveDistance > 12 then
			StringBuffer.Append(oorange," HomunMaxMoveDistance")
			HomunMaxMoveDistance = 12
		end
	end

	-- Verify EnableIdleMove
	if type(EnableIdleMove) ~= "boolean" then
		StringBuffer.Append(invalids," EnableIdleMove")
		EnableIdleMove = true
	end

	-- Verify IdleMoveRadius
	if type(IdleMoveRadius) ~= "number" then
		StringBuffer.Append(invalids," IdleMoveRadius")
		IdleMoveRadius = 5
	else
		-- Check range
		if IdleMoveRadius < 2 then
			StringBuffer.Append(oorange," IdleMoveRadius")
			IdleMoveRadius = 2
		elseif IdleMoveRadius > 6 then
			StringBuffer.Append(oorange," IdleMoveRadius")
			IdleMoveRadius = 6
		end
	end

	-- Verify TimeBeforeIdle
	if type(TimeBeforeIdle) ~= "number" then
		StringBuffer.Append(invalids," TimeBeforeIdle")
		TimeBeforeIdle = 1000
	else
		-- Check the bottom limit
		if TimeBeforeIdle < 0 then
			StringBuffer.Append(oorange," TimeBeforeIdle")
			TimeBeforeIdle = 0
		end
	end

	--[[
		=== Interjected PercentHP and PercentSP Verification
		=== Here, For Immediate Use
	--]]
	
	-- Verify PercentHP
	if type(PercentHP) ~= "boolean" then
		StringBuffer.Append(invalids," PercentHP")
		PercentHP = true
	end
	
	-- Verify PercentSP
	if type(PercentSP) ~= "boolean" then
		StringBuffer.Append(invalids," PercentSP")
		PercentSP = false
	end
	
	--[[
		=== Back to Following Config.lua Order
	--]]
	
	-- Verify IdleMoveHP
	if type(IdleMoveHP) ~= "number" then
		StringBuffer.Append(invalids," IdleMoveHP")
		IdleMoveHP = 50
	else
		-- Check the range
		if IdleMoveHP < 0 then
			StringBuffer.Append(oorange," IdleMoveHP")
			IdleMoveHP = 0
		elseif IdleMoveHP > 100 and PercentHP then
			StringBuffer.Append(oorange," IdleMoveHP")
			IdleMoveHP = 100
		end
	end

	-- Verify IdleMoveSP
	if type(IdleMoveSP) ~= "number" then
		StringBuffer.Append(invalids," IdleMoveSP")
		IdleMoveSP = 50
	else
		-- Check the range
		if IdleMoveSP < 0 then
			StringBuffer.Append(oorange," IdleMoveSP")
			IdleMoveSP = 0
		elseif IdleMoveSP > 100 and PercentSP then
			StringBuffer.Append(oorange," IdleMoveSP")
			IdleMoveSP = 100
		end
	end

	-- Verify EnableAMC
	if type(EnableAMC) ~= "boolean" then
		StringBuffer.Append(invalids," EnableAMC")
		EnableAMC = true
	end
	
	-- Verify QuickdialAMC
	if type(QuickdialAMC) ~= "boolean" then
		StringBuffer.Append(invalids," QuickdialAMC")
		QuickdialAMC = true
	end
	
	-- Verify AMCResetTime
	if type(AMCResetTime) ~= "number" then
		StringBuffer.Append(invalids," AMCResetTime")
		AMCResetTime = 5000
	else
		-- Verify the bottom limit of AMCResetTime
		if AMCResetTime < 100 then
			AMCResetTime = 100
		end
	end
	
	-- Verify DefaultAggro
	if type(DefaultAggro) ~= "boolean" then
		StringBuffer.Append(invalids," DefaultAggro")
		DefaultAggro = false
	end
	
	-- Verify AssistOwner
	if type(AssistOwner) ~= "boolean" then
		StringBuffer.Append(invalids," AssistOwner")
		AssistOwner = true
	end
	
	-- Verify AllowKS
	if type(AllowKS) ~= "boolean" then
		StringBuffer.Append(invalids," AllowKS")
		AllowKS = false
	end

	-- Verify ActorIgnoreTime
	if type(ActorIgnoreTime) ~= "number" then
		StringBuffer.Append(invalids," ActorIgnoreTime")
		ActorIgnoreTime = 5000
	else
		-- Verify the bottom limit
		if ActorIgnoreTime < 0 then
			ActorIgnoreTime = 0
		end
	end

	-- Verify EnableIgnoreAfterTicks
	if type(EnableIgnoreAfterTicks) ~= "boolean" then
		StringBuffer.Append(invalids," EnableIgnoreAfterTicks")
		EnableIgnoreAfterTicks = false
	end

	-- Verify ActorIgnoreAfterTicks
	if type(ActorIgnoreAfterTicks) ~= "number" then
		StringBuffer.Append(invalids," ActorIgnoreAfterTicks")
		ActorIgnoreAfterTicks = 30000
	else
		-- Check the bottom limit
		if ActorIgnoreAfterTicks < 5000 then
			StringBuffer.Append(oorange," ActorIgnoreAfterTicks")
			ActorIgnoreAfterTicks = 5000
		end
	end
	
	-- Verify AcquireTargetTimeout
	if type(AcquireTargetTimeout) ~= "number" then
		StringBuffer.Append(invalids," AcquireTargetTimeout")
		AcquireTargetTimeout = 50
	else
		-- Verify the bottom limit
		if AcquireTargetTimeout < 50 then
			AcquireTargetTimeout = 50
		end
	end
	
	-- Verify AcquireWhileLocked
	if type(AcquireWhileLocked) ~= "boolean" then
		StringBuffer.Append(invalids," AcquireWhileLocked")
		AcquireWhileLocked = false
	end

	-- Verify EnableAttacksTimeout
	if type(EnableAttacksTimeout) ~= "boolean" then
		StringBuffer.Append(invalids," EnableAttacksTimeout")
		EnableAttacksTimeout = true
	end

	-- Verify AttacksTimeout
	if type(AttacksTimeout) ~= "number" then
		StringBuffer.Append(invalids," AttacksTimeout")
		AttacksTimeout = 1500
	else
		if AttacksTimeout < 250 then
			StringBuffer.Append(oorange," AttacksTimeout")
			AttacksTimeout = 250
		end
	end

	-- Verify AttacksMaxTries
	if type(AttacksMaxTries) ~= "number" then
		StringBuffer.Append(invalids," AttacksMaxTries")
		AttacksMaxTries = 5
	else
		if AttacksMaxTries < 1 then
			StringBuffer.Append(oorange," AttacksMaxTries")
			AttacksMaxTries = 1
		end
	end

	-- Verify KiteDefaultDistance
	if type(KiteDefaultDistance) ~= "number" then
		StringBuffer.Append(invalids," KiteDefaultDistance")
		KiteDefaultDistance = 6
	else
		-- Check the range
		if KiteDefaultDistance < 2 then
			StringBuffer.Append(oorange," KiteDefaultDistance")
			KiteDefaultDistance = 2
		elseif KiteDefaultDistance > MaxHomunRange - 4 then
			StringBuffer.Append(oorange," KiteDefaultDistance")
			KiteDefaultDistance = MaxHomunRange - 4
		end
	end

	-- Verify PassiveDefense
	if type(PassiveDefense) ~= "boolean" then
		StringBuffer.Append(invalids," PassiveDefense")
		PassiveDefense = true
	end
	
	-- Verify PassiveSkills
	if type(PassiveSkills) ~= "boolean" then
		StringBuffer.Append(invalids," PassiveSkills")
		PassiveSkills = true
	end
	
	-- Verify PrioritizeOwnerDefense (range verified during init)
	if type(PrioritizeOwnerDefense) ~= "number" then
		StringBuffer.Append(invalids," PrioritizeOwnerDefense")
		PrioritizeOwnerDefense = 1
	end
	
	-- Verify PrioritizeHomunDefense (range verified during init)
	if type(PrioritizeHomunDefense) ~= "number" then
		StringBuffer.Append(invalids," PrioritizeHomunDefense")
		PrioritizeHomunDefense = 5
	end

	-- Verify PrioritizeFriendDefense (range verified during init)
	if type(PrioritizeFriendDefense) ~= "number" then
		StringBuffer.Append(invalids," PrioritizeFriendDefense")
		PrioritizeFriendDefense = 4
	end

	-- Verify DefendFriends
	if type(DefendFriends) ~= "boolean" then
		StringBuffer.Append(invalids," DefendFriends")
		DefendFriends = true
	end

	-- Verify TemporaryFriendTiles
	if type(TemporaryFriendTiles) ~= "number" then
		StringBuffer.Append(invalids," TemporaryFriendTiles")
		TemporaryFriendTiles = -1
	end

	-- Verify AutoSaveFriends
	if type(AutoSaveFriends) ~= "boolean" then
		StringBuffer.Append(invalids," AutoSaveFriends")
		AutoSaveFriends = false
	end
	
	-- Verify AllowSkills
	if type(AllowSkills) ~= "boolean" then
		StringBuffer.Append(invalids," AllowSkills")
		AllowSkills = true
	end
	
	-- Verify SkillsCauseAttack
	if type(SkillsCauseAttack) ~= "boolean" then
		StringBuffer.Append(invalids," SkillsCauseAttack")
		SkillsCauseAttack = false
	end
	
	-- Verify MinSP
	if type(MinSP) ~= "number" then
		StringBuffer.Append(invalids," MinSP")
		MinSP = 0
	else
		-- Verify the range of MinSP
		if MinSP < 0 then
			StringBuffer.Append(oorange," MinSP")
			MinSP = 0
		elseif MinSP > 100 and PercentSP then
			StringBuffer.Append(oorange," MinSP")
			MinSP = 100
		end
	end

	-- Verify IgnoreAttackSkillRange
	if type(IgnoreAttackSkillRange) ~= "boolean" then
		StringBuffer.Append(invalids," IgnoreAttackSkillRange")
		IgnoreAttackSkillRange = false
	end

	-- Verify IgnoredRangeFromOwner
	if type(IgnoredRangeFromOwner) ~= "number" then
		StringBuffer.Append(invalids," IgnoredRangeFromOwner")
		IgnoredRangeFromOwner = 15
	end
	
	-- Verify MinAttackSkillLevel
	if type(MinAttackSkillLevel) ~= "number" then
		StringBuffer.Append(invalids," MinAttackSkillLevel")
		MinAttackSkillLevel = 1
	else
		-- Verify the range
		if MinAttackSkillLevel < 1 then
			StringBuffer.Append(oorange," MinAttackSkillLevel")
			MinAttackSkillLevel = 1
		elseif MinAttackSkillLevel > 5 then
			StringBuffer.Append(oorange," MinAttackSkillLevel")
			MinAttackSkillLevel = 5
		end
	end
	
	-- Verify MeleeSkills
	if type(MeleeSkills) ~= "boolean" then
		StringBuffer.Append(invalids," MeleeSkills")
		MeleeSkills = true
	end
	
	-- Verify ChaseSkills
	if type(ChaseSkills) ~= "boolean" then
		StringBuffer.Append(invalids," ChaseSkills")
		ChaseSkills = true
	end

	-- Verify LockOnAttackSkills
	if type(LockOnAttackSkills) ~= "boolean" then
		StringBuffer.Append(invalids," LockOnAttackSkills")
		LockOnAttackSkills = false
	end
	
	-- Verify SupportSkills
	if type(SupportSkills) ~= "boolean" then
		StringBuffer.Append(invalids," SupportSkills")
		SupportSkills = true
	end
	
	-- Verify HealSkills
	if type(HealSkills) ~= "boolean" then
		StringBuffer.Append(invalids," HealSkills")
		HealSkills = true
	end

	-- Verify AlchTargetSeparate
	if type(AlchTargetSeparate) ~= "boolean" then
		StringBuffer.Append(invalids," AlchTargetSeparate")
		AlchTargetSeparate = false
	end

	-- Verify AlchTargetForceDifferent
	if type(AlchTargetForceDifferent) ~= "boolean" then
		StringBuffer.Append(invalids," AlchTargetForceDifferent")
		AlchTargetForceDifferent = false
	end

	-- Verify AlchTargetSameWhenHP
	if type(AlchTargetSameWhenHP) ~= "number" then
		StringBuffer.Append(invalids," AlchTargetSameWhenHP")
		AlchTargetSameWhenHP = 0
	else
		-- Check the range
		if AlchTargetSameWhenHP < 0 then
			StringBuffer.Append(oorange," AlchTargetSameWhenHP")
			AlchTargetSameWhenHP = 0
		elseif AlchTargetSameWhenHP > 100 and PercentHP then
			StringBuffer.Append(oorange," AlchTargetSameWhenHP")
			AlchTargetSameWhenHP = 100
		end
	end

	-- Verify AutoAlchSkillMinLevel
	if type(AutoAlchSkillMinLevel) ~= "number" then
		StringBuffer.Append(invalids," AutoAlchSkillMinLevel")
		AutoAlchSkillMinLevel = 1
	else
		-- Check the range
		if AutoAlchSkillMinLevel < 1 then
			StringBuffer.Append(oorange," AutoAlchSkillMinLevel")
			AutoAlchSkillMinLevel = 1
		elseif AutoAlchSkillMinLevel > 5 then
			StringBuffer.Append(oorange," AutoAlchSkillMinLevel")
			AutoAlchSkillMinLevel = 5
		end
	end

	-- Verify AutoAlchSkillDelay
	if type(AutoAlchSkillDelay) ~= "number" then
		StringBuffer.Append(invalids," AutoAlchSkillDelay")
		AutoAlchSkillDelay = 200
	else
		-- Check the bottom limit
		if AutoAlchSkillDelay < 0 then
			StringBuffer.Append(oorange," AutoAlchSkillDelay")
			AutoAlchSkillDelay = 0
		end
	end

	-- Verify AutoSplashCart
	if type(AutoSplashCart) ~= "boolean" then
		StringBuffer.Append(invalids," AutoSplashCart")
		AutoSplashCart = false
	end

	-- Verify AutoSplashBlast
	if type(AutoSplashBlast) ~= "boolean" then
		StringBuffer.Append(invalids," AutoSplashBlast")
		AutoSplashBlast = false
	end

	-- Verify AutoSplashBlastMinLvl
	if type(AutoSplashBlastMinLvl) ~= "number" then
		StringBuffer.Append(invalids," AutoSplashBlastMinLvl")
		AutoSplashBlastMinLvl = 1
	else
		-- Check the range
		if AutoSplashBlastMinLvl < 1 then
			StringBuffer.Append(oorange," AutoSplashBlastMinLvl")
			AutoSplashBlastMinLvl = 1
		elseif AutoSplashBlastMinLvl > 3 then
			StringBuffer.Append(oorange," AutoSplashBlastMinLvl")
			AutoSplashBlastMinLvl = 3
		end
	end

	-- Verify AutoSplashMinHits
	if type(AutoSplashMinHits) ~= "number" then
		StringBuffer.Append(invalids," AutoSplashMinHits")
		AutoSplashMinHits = 1
	else
		-- Check the bottom range
		if AutoSplashMinHits < 1 then
			StringBuffer.Append(oorange," AutoSplashMinHits")
			AutoSplashMinHits = 1
		end
	end

	-- Verify AutoSplashTimeout
	if type(AutoSplashTimeout) ~= "number" then
		StringBuffer.Append(invalids," AutoSplashTimeout")
		AutoSplashTimeout = 1000
	else
		-- Check the bottom range
		if AutoSplashTimeout < 0 then
			StringBuffer.Append(oorange," AutoSplashTimeout")
			AutoSplashTimeout = 0
		end
	end

	-- Verify AutoHealFriends
	if type(AutoHealFriends) ~= "number" then
		StringBuffer.Append(invalids," AutoHealFriends")
		AutoHealFriends = 0
	else
		-- Check the bottom range
		if AutoHealFriends < 0 then
			StringBuffer.Append(oorange," AutoHealFriends")
			AutoHealFriends = 0
		end
	end

	-- Verify AutoHealTempFriends
	if type(AutoHealTempFriends) ~= "number" then
		StringBuffer.Append(invalids," AutoHealTempFriends")
		AutoHealTempFriends = 0
	else
		-- Check the bottom range
		if AutoHealTempFriends < 0 then
			StringBuffer.Append(oorange," AutoHealTempFriends")
			AutoHealTempFriends = 0
		end
	end

	-- Verify AutoHealAnybody
	if type(AutoHealAnybody) ~= "number" then
		StringBuffer.Append(invalids," AutoHealAnybody")
		AutoHealAnybody = 0
	else
		-- Check the bottom range
		if AutoHealAnybody < 0 then
			StringBuffer.Append(oorange," AutoHealAnybody")
			AutoHealAnybody = 0
		end
	end

	-- Verify AutoHealHitTimeouts
	if type(AutoHealHitTimeouts) ~= "number" then
		StringBuffer.Append(invalids," AutoHealHitTimeouts")
		AutoHealHitTimeouts = 5000
	else
		-- Check the bottom limit
		if AutoHealHitTimeouts < 0 then
			StringBuffer.Append(oorange," AutoHealHitTimeouts")
			AutoHealHitTimeouts = 0
		end
	end

	-- Verify AutoHealMinLevel
	if type(AutoHealMinLevel) ~= "number" then
		StringBuffer.Append(invalids," AutoHealMinLevel")
		AutoHealMinLevel = 1
	else
		-- Check the range
		if AutoHealMinLevel < 1 then
			StringBuffer.Append(oorange," AutoHealMinLevel")
			AutoHealMinLevel = 1
		elseif AutoHealMinLevel > 3 then
			StringBuffer.Append(oorange," AutoHealMinLevel")
			AutoHealMinLevel = 3
		end
	end

	-- Verify AutoHealBomb
	if type(AutoHealBomb) ~= "boolean" then
		StringBuffer.Append(invalids," AutoHealBomb")
		AutoHealBomb = false
	end

	-- Verify AutoAcidDemo
	if type(AutoAcidDemo) ~= "boolean" then
		StringBuffer.Append(invalids," AutoAcidDemo")
		AutoAcidDemo = false
	end

	-- Verify AutoAcidTerror
	if type(AutoAcidTerror) ~= "boolean" then
		StringBuffer.Append(invalids," AutoAcidTerror")
		AutoAcidTerror = false
	end

	-- Verify AutoBoltSword
	if type(AutoBoltSword) ~= "boolean" then
		StringBuffer.Append(invalids," AutoBoltSword")
		AutoBoltSword = false
	end

	-- Verify AutoCutlusBash
	if type(AutoCutlusBash) ~= "boolean" then
		StringBuffer.Append(invalids," AutoCutlusBash")
		AutoCutlusBash = false
	end

	-- Verify AutoTomahawk
	if type(AutoTomahawk) ~= "boolean" then
		StringBuffer.Append(invalids," AutoTomahawk")
		AutoTomahawk = false
	end

	-- Verify AutoAidPot
	if type(AutoAidPot) ~= "boolean" then
		StringBuffer.Append(invalids," AutoAidPot")
		AutoAidPot = false
	end
	
	-- Verify AidPotHP
	if type(AidPotHP) ~= "number" then
		StringBuffer.Append(invalids," AidPotHP")
		AidPotHP = 25
	else
		-- Verify the range of AidPotHP
		if AidPotHP < 0 then
			StringBuffer.Append(oorange," AidPotHP")
			AidPotHP = 0
		elseif AidPotHP > 100 and PercentHP then
			StringBuffer.Append(oorange," AidPotHP")
			AidPotHP = 100
		end
	end

	-- Verify AidPotLevel
	if type(AidPotLevel) ~= "number" then
		StringBuffer.Append(invalids," AidPotLevel")
		AidPotLevel = 4
	else
		-- Verify the range of AidPotLevel
		if AidPotLevel < 1 then
			StringBuffer.Append(oorange," AidPotLevel")
			AidPotLevel = 1
		elseif AidPotLevel > 4 then
			StringBuffer.Append(oorange," AidPotLevel")
			AidPotLevel = 4
		end
	end

	-- Verify AidPotMoveCloser
	if type(AidPotMoveCloser) ~= "boolean" then
		StringBuffer.Append(invalids," AidPotMoveCloser")
		AidPotMoveCloser = true
	end

	-- Verify AidPotWaitAfterMove
	if type(AidPotWaitAfterMove) ~= "number" then
		StringBuffer.Append(invalids," AidPotWaitAfterMove")
		AidPotWaitAfterMove = 500
	else
		-- Verify the bottom limit
		if AidPotWaitAfterMove < 0 then
			StringBuffer.Append(oorange," AidPotWaitAfterMove")
			AidPotWaitAfterMove = 0
		end
	end

	-- Verify AutoAidBlue
	if type(AutoAidBlue) ~= "boolean" then
		StringBuffer.Append(invalids," AutoAidBlue")
		AutoAidBlue = false
	end

	-- Verify AidBlueSP
	if type(AidBlueSP) ~= "number" then
		StringBuffer.Append(invalids," AidBlueSP")
		AidBlueSP = 25
	else
		-- Verify the range
		if AidBlueSP < 0 then
			StringBuffer.Append(oorange," AidBlueSP")
			AidBlueSP = 0
		elseif AidBlueSP > 100 and PercentSP then
			StringBuffer.Append(oorange," AidBlueSP")
			AidBlueSP = 100
		end
	end

	-- Verify AidPotAdvanced
	if type(AidPotAdvanced) ~= "boolean" then
		StringBuffer.Append(invalids," AidPotAdvanced")
		AidPotAdvanced = false
	end

	-- Verify AidPotLowest
	if type(AidPotLowest) ~= "number" then
		StringBuffer.Append(invalids," AidPotLowest")
		AidPotLowest = 3
	else
		-- Verify the range of AidPotLowest
		if AidPotLowest < 1 then
			StringBuffer.Append(oorange," AidPotLowest")
			AidPotLowest = 1
		elseif AidPotLowest > 4 then
			StringBuffer.Append(oorange," AidPotLowest")
			AidPotLowest = 4
		end
	end

	-- Verify AidPotHighest
	if type(AidPotHighest) ~= "number" then
		StringBuffer.Append(invalids," AidPotHighest")
		AidPotHighest = 4
	else
		-- Verify the range
		if AidPotHighest < 1 then
			StringBuffer.Append(oorange," AidPotHighest")
			AidPotHighest = 1
		elseif AidPotHighest > 4 then
			StringBuffer.Append(oorange," AidPotHighest")
			AidPotHighest = 4
		end
	end

	-- Verify AidPotIncrement
	if type(AidPotIncrement) ~= "number" then
		StringBuffer.Append(invalids," AidPotIncrement")
		AidPotIncrement = 3
	else
		-- Verify the bottom limit
		if AidPotIncrement < 0 then
			AidPotIncrement = 0
		end
	end
	
	-- Verify AidPotTriesPerLevel
	if type(AidPotTriesPerLevel) ~= "number" then
		StringBuffer.Append(invalids," AidPotTriesPerLevel")
		AidPotTriesPerLevel = 3
	else
		-- Verify the bottom limit
		if AidPotTriesPerLevel < 1 then
			AidPotTriesPerLevel = 1
		end
	end
	
	-- Verify AidPotTriesReset
	if type(AidPotTriesReset) ~= "number" then
		StringBuffer.Append(invalids," AidPotTriesReset")
		AidPotTriesReset = 60000
	else
		-- Verify the bottom limit
		if AidPotTriesReset < 50 then
			AidPotTriesReset = 50
		end
	end
	
	-- Verify AidPotWaitTime
	if type(AidPotWaitTime) ~= "number" then
		StringBuffer.Append(invalids," AidPotWaitTime")
		AidPotWaitTime = 200
	else
		-- Verify the bottom limit
		if AidPotWaitTime < 50 then
			AidPotWaitTime = 50
		end
	end

	-- Verify AutoRes
	if type(AutoRes) ~= "boolean" then
		StringBuffer.Append(invalids," AutoRes")
		AutoRes = false
	end

	-- Verify AutoResHP
	if type(AutoResHP) ~= "number" then
		StringBuffer.Append(invalids," AutoResHP")
		AutoResHP = 15
	else
		-- Verify the range of AutoResHP
		if AutoResHP < 0 then
			StringBuffer.Append(oorange," AutoResHP")
			AutoResHP = 0
		elseif AutoResHP > 100 and PercentHP then
			StringBuffer.Append(oorange," AutoResHP")
			AutoResHP = 100
		end
	end

	-- Verify AutoResSmart
	if type(AutoResSmart) ~= "boolean" then
		StringBuffer.Append(invalids," AutoResSmart")
		AutoResSmart = true
	end

	-- Verify AutoResDelay
	if type(AutoResDelay) ~= "number" then
		StringBuffer.Append(invalids," AutoResDelay")
		AutoResDelay = 400
	else
		-- Check the bottom limit
		if AutoResDelay < 0 then
			StringBuffer.Append(oorange," AutoResDelay")
			AutoResDelay = 0
		end
	end
	
	-- Verify AmistrBulwarkOwner
	if type(AmistrBulwarkOwner) ~= "number" then
		StringBuffer.Append(invalids," AmistrBulwarkOwner")
		AmistrBulwarkOwner = 3
	else
		-- Verify the bottom limit
		if AmistrBulwarkOwner < 0 then
			AmistrBulwarkOwner = 0
		end
	end
	
	-- Verify AmistrBulwarkHomun
	if type(AmistrBulwarkHomun) ~= "number" then
		StringBuffer.Append(invalids," AmistrBulwarkHomun")
		AmistrBulwarkHomun = 6
	else
		-- Verify the bottom limit
		if AmistrBulwarkHomun < 0 then
			AmistrBulwarkHomun = 0
		end
	end
	
	-- Verify AmistrBulwarkMinLevel
	if type(AmistrBulwarkMinLevel) ~= "number" then
		StringBuffer.Append(invalids," AmistrBulwarkMinLevel")
		AmistrBulwarkMinLevel = 1
	else
		-- Verify the range
		if AmistrBulwarkMinLevel < 1 then
			StringBuffer.Append(oorange," AmistrBulwarkMinLevel")
			AmistrBulwarkMinLevel = 1
		elseif AmistrBulwarkMinLevel > 5 then
			StringBuffer.Append(oorange," AmistrBulwarkMinLevel")
			AmistrBulwarkMinLevel = 5
		end
	end
	
	-- Verify CastlingAttackers
	if type(CastlingAttackers) ~= "number" then
		StringBuffer.Append(invalids," CastlingAttackers")
		CastlingAttackers = 5
	else
		-- Check the bottom limit
		if CastlingAttackers < 0 then
			StringBuffer.Append(oorange," CastlingAttackers")
			CastlingAttackers = 0
		end
	end
	
	-- Verify CastlingHP
	if type(CastlingHP) ~= "number" then
		StringBuffer.Append(invalids," CastlingHP")
		CastlingHP = 100
	else
		-- Check the range
		if CastlingHP < 0 then
			StringBuffer.Append(oorange," CastlingHP")
			CastlingHP = 0
		elseif CastlingHP > 100 and PercentHP then
			StringBuffer.Append(oorange," CastlingHP")
			CastlingHP = 100
		end
	end

	-- Verify CastlingDelay
	if type(CastlingDelay) ~= "number" then
		StringBuffer.Append(invalids," CastlingDelay")
		CastlingDelay = 500
	else
		-- Check the range
		if CastlingDelay < 0 then
			StringBuffer.Append(oorange," CastlingDelay")
			CastlingDelay = 0
		end
	end

	-- Verify FilirUseDuration
	if type(FilirUseDuration) ~= "boolean" then
		StringBuffer.Append(invalids," FilirUseDuration")
		FilirUseDuration = true
	end

	-- Verify FlittingLevel
	if type(FlittingLevel) ~= "number" then
		StringBuffer.Append(invalids," FlittingLevel")
		FlittingLevel = 5
	else
		-- Check the range
		if FlittingLevel < 0 then
			StringBuffer.Append(oorange," FlittingLevel")
			FlittingLevel = 0
		elseif FlittingLevel > 5 then
			StringBuffer.Append(oorange," FlittingLevel")
			FlittingLevel = 5
		end
	end

	-- Verify FlittingMinTicks
	if type(FlittingMinTicks) ~= "number" then
		StringBuffer.Append(invalids," FlittingMinTicks")
		FlittingMinTicks = 0
	end

	-- Verify FlittingSPSaveTicks
	if type(FlittingSPSaveTicks) ~= "number" then
		StringBuffer.Append(invalids," FlittingSPSaveTicks")
		FlittingSPSaveTicks = 20000
	else
		-- Check the bottom
		if FlittingSPSaveTicks < 0 then
			StringBuffer.Append(oorange," FlittingSPSaveTicks")
			FlittingSPSaveTicks = 0
		end
	end

	-- Verify MinHealingHands
	if type(MinHealingHands) ~= "number" then
		StringBuffer.Append(invalids," MinHealingHands")
		MinHealingHands = 1
	else
		-- Change range
		if MinHealingHands < 1 then
			StringBuffer.Append(oorange," MinHealingHands")
			MinHealingHands = 1
		elseif MinHealingHands > 5 then
			StringBuffer.Append(oorange," MinHealingHands")
			MinHealingHands = 5
		end
	end

	-- Verify HealingHandsHP
	if type(HealingHandsHP) ~= "number" then
		StringBuffer.Append(invalids," HealingHandsHP")
		HealingHandsHP = 50
	else
		-- Check the range
		if HealingHandsHP < 0 then
			StringBuffer.Append(oorange," HealingHandsHP")
			HealingHandsHP = 0
		elseif HealingHandsHP > 100 and PercentHP then
			StringBuffer.Append(oorange," HealingHandsHP")
			HealingHandsHP = 100
		end
	end

	-- Verify HealingHandsMinEnemies
	if type(HealingHandsMinEnemies) ~= "number" then
		StringBuffer.Append(invalids," HealingHandsMinEnemies")
		HealingHandsMinEnemies = 0
	else
		-- Check the bottom limit
		if HealingHandsMinEnemies < 0 then
			StringBuffer.Append(oorange," HealingHandsMinEnemies")
			HealingHandsMinEnemies = 0
		end
	end

	-- Verify ChaoticBlessingsOwnerHP
	if type(ChaoticBlessingsOwnerHP) ~= "number" then
		StringBuffer.Append(invalids," ChaoticBlessingsOwnerHP")
		ChaoticBlessingsOwnerHP = 50
	else
		-- Check the range
		if ChaoticBlessingsOwnerHP < 0 then
			StringBuffer.Append(oorange," ChaoticBlessingsOwnerHP")
			ChaoticBlessingsOwnerHP = 0
		elseif ChaoticBlessingsOwnerHP > 100 and PercentHP then
			StringBuffer.Append(oorange," ChaoticBlessingsOwnerHP")
			ChaoticBlessingsOwnerHP = 100
		end
	end

	-- Verify ChaoticBlessingsHomunHP
	if type(ChaoticBlessingsHomunHP) ~= "number" then
		StringBuffer.Append(invalids," ChaoticBlessingsHomunHP")
		ChaoticBlessingsHomunHP = 0
	else
		-- Check the range
		if ChaoticBlessingsHomunHP < 0 then
			StringBuffer.Append(oorange," ChaoticBlessingsHomunHP")
			ChaoticBlessingsHomunHP = 0
		elseif ChaoticBlessingsHomunHP > 100 and PercentHP then
			StringBuffer.Append(oorange," ChaoticBlessingsHomunHP")
			ChaoticBlessingsHomunHP = 100
		end
	end

	-- Verify MaxChaoticBlessings
	if type(MaxChaoticBlessings) ~= "number" then
		StringBuffer.Append(invalids," MaxChaoticBlessings")
		MaxChaoticBlessings = 5
	else
		-- Check the range
		if MaxChaoticBlessings < 1 then
			StringBuffer.Append(oorange," MaxChaoticBlessings")
			MaxChaoticBlessings = 1
		elseif MaxChaoticBlessings > 5 then
			StringBuffer.Append(oorange," MaxChaoticBlessings")
			MaxChaoticBlessings = 5
		end
	end

	-- Verify ChaoticBlessingsWhileLocked
	if type(ChaoticBlessingsWhileLocked) ~= "boolean" then
		StringBuffer.Append(invalids," ChaoticBlessingsWhileLocked")
		ChaoticBlessingsWhileLocked = false
	end

	-- Verify AntiPosLag
	if type(AntiPosLag) ~= "boolean" then
		StringBuffer.Append(invalids," AntiPosLag")
		AntiPosLag = true
	end

	-- Verify PosLagTries
	if type(PosLagTries) ~= "number" then
		StringBuffer.Append(invalids," PosLagTries")
		PosLagTries = 3
	else
		-- Check the bottom limit
		if PosLagTries < 1 then
			StringBuffer.Append(oorange," PosLagTries")
			PosLagTries = 1
		end
	end

	-- Verify PosLagDelay
	if type(PosLagDelay) ~= "number" then
		StringBuffer.Append(invalids," PosLagDelay")
		PosLagDelay = 500
	else
		-- Check the bottom limit
		if PosLagDelay < 0 then
			StringBuffer.Append(oorange," PosLagDelay")
			PosLagDelay = 50
		end
	end

	-- Verify OwnerMinSP
	if type(OwnerMinSP) ~= "number" then
		StringBuffer.Append(invalids," OwnerMinSP")
		OwnerMinSP = 0
	else
		-- Check the range on it
		if OwnerMinSP < 0 then
			StringBuffer.Append(oorange," OwnerMinSP")
			OwnerMinSP = 0
		elseif OwnerMinSP > 100 and PercentSP then
			StringBuffer.Append(oorange," OwnerMinSP")
			OwnerMinSP = 100
		end
	end

	--[[
	
		NOTE: PercentHP and PercentSP moved farther up
	
	--]]

	-- Verify AIStartWait
	if type(AIStartWait) ~= "number" then
		StringBuffer.Append(invalids," AIStartWait")
		AIStartWait = 1000
	else
		-- Check the bottom limit of it
		if AIStartWait < 0 then
			StringBuffer.Append(oorange," AIStartWait")
			AIStartWait = 0
		end
	end

	-- Verify AssumePVP
	if type(AssumePVP) ~= "boolean" then
		StringBuffer.Append(invalids," AssumePVP")
		AssumePVP = false
	end

	-- Verify AssumeWoE
	if type(AssumeWoE) ~= "boolean" then
		StringBuffer.Append(invalids," AssumeWoE")
		AssumeWoE = false
	end

	-- Verify CheckPVP
	if type(CheckPVP) ~= "boolean" then
		StringBuffer.Append(invalids," CheckPVP")
		CheckPVP = true
	end
	
	-- Verify MoveIsAttackWhenDistance
	if type(MoveIsAttackWhenDistance) ~= "number" then
		StringBuffer.Append(invalids," MoveIsAttackWhenDistance")
		MoveIsAttackWhenDistance = 0
	else
		-- Verify the range
		if MoveIsAttackWhenDistance < 0 then
			StringBuffer.Append(oorange," MoveIsAttackWhenDistance")
			MoveIsAttackWhenDistance = 0
		elseif MoveIsAttackWhenDistance > 5 then
			StringBuffer.Append(oorange," MoveIsAttackWhenDistance")
			MoveIsAttackWhenDistance = 5
		end
	end
	
	-------------------------------
	-- Check if any were invalid --
	-------------------------------
	
	if List.size(invalids) > 0 then
		-- Output all the ones that were invalid
		Log(0,"Invalid configuration values for the following options:%s",StringBuffer.Get(invalids))
	end

	if List.size(oorange) > 0 then
		-- Output all the options that were out of range
		Log(0,"The following options had values that were out of range:%s",StringBuffer.Get(oorange))
	end
end
