--[[

This file is part of RampageAI.

RampageAI is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

RampageAI is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with RampageAI; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

--]]

------------------
-- Table Object --
------------------

Table = { }

-- Create a new table
function Table.new()
	return { n=0 }
end

-- Get table size
function Table.getn(t)
	-- See if the table has an "n" element
	if t.n == nil then
		-- Otherwise count until we find nil
		local i = 0
		while t[i+1] ~= nil do i=i+1 end
		t.n = i
	end

	return t.n
end

-- Set the table size
function Table.setn(t,size)
	-- Get the table size
	local len = Table.getn(t)
	
	-- Check if we're reducing the size
	if size < len then
		-- Remove all elements after size
		local i
		for i=len,size+1,-1 do
			t[i]=nil
		end
	end

	-- Set n
	t.n=size
end

-- Add an element to a table
function Table.insert(t,pos,item)
	-- Check if we were given 3 arguments
	if item == nil then
		-- Nope, only 2, 2nd one is the item
		item = pos
		pos = nil
	end

	-- Get the number of items we're adding
	local num = 1
	if type(item) == "table" and item.n ~= nil then
		num = Table.getn(item)
	end

	-- Save the number of items in t
	local tEnd = Table.getn(t)

	-- Increase number of items
	t.n = Table.getn(t) + num

	-- Check if we have a position
	if pos == nil then
		-- Set pos to the end
		pos = tEnd + 1
	end

	-- Shift everything forward
	local i
	for i=t.n,pos+num,-1 do
		t[i]=t[i-num]
	end

	-- Insert the item(s)
	if type(item) == "table" and item.n ~= nil then
		-- Loop through the table to be merged
		for i=1,Table.getn(item) do
			-- Merge them in
			t[pos+i-1]=item[i]
		end
	else
		t[pos]=item
	end
end

-- Remove an element from a table
function Table.remove(t,pos,num)
	-- Check if we have num
	if num == nil or num < 0 then
		num = 1
	elseif num == 0 then
		return
	end

	-- Check if the item exists
	if Table.getn(t) < pos then
		return
	end

	-- Shift everythign forward
	local i
	for i=pos,t.n-num do
		t[i]=t[i+num]
		t[i+num]=nil
	end

	-- Remove the previous elements
	t.n=t.n-num
end

-- Remove elements from the table, based on a function evaluating their values
function Table.sieveValues(t,f,reversed)
	-- Make sure we have a sieving function
	if type(f) ~= "function" then
		return t
	end

	-- Make sure reversed is a boolean
	if type(reversed) ~= "boolean" then
		reversed = false
	end

	-- Loop through each element in the table
	local i,max = 1,Table.getn(t)
	while i <= max do
		-- Check if the element should be ignored
		local ret = f(t[i])
		if type(ret) ~= "boolean" then
			ret = false
		end
		if ret ~= reversed then
			-- Element should be removed
			Table.remove(t,i)

			-- Reduce the max
			max = Table.getn(t)
		else
			-- Element should stay
			i = i + 1
		end
	end

	return t
end
